#ifndef _TransparentWidget_h_
#define _TransparentWidget_h_
// $Id: TransparentWidget.h,v 1.22 2009/09/09 02:29:38 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file    TransparentWidget.h
  \brief   transparent widget
  \author  Hugo Pereira
  \version $Revision: 1.22 $
  \date    $Date: 2009/09/09 02:29:38 $
*/

#include <QAction>
#include <QMoveEvent>
#include <QPaintEvent>
#include <QResizeEvent>
#include <QShowEvent>
#include <QWidget>

#include "Counter.h"

//! clock widgets
namespace TRANSPARENCY {

  //! transparent widget
  class TransparentWidget: public QWidget, public Counter
  {

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    TransparentWidget( QWidget *parent = 0, Qt::WindowFlags flags = 0 );

    //!@name actions
    //@{

    //! update background
    QAction& updateBackgroundAction( void ) const
    { return *update_background_action_; }

    //! reload background action
    QAction& reloadBackgroundAction( void ) const
    { return *reload_background_action_; }

    //@}

    //! window opacity
    /*! this overloads the QWidget method to hack for WIN system */
    void setWindowOpacity( double );

    //! background changed
    void setBackgroundChanged( const bool& value )
    { background_changed_ = value; }

    public slots:

    //! force reloading of the background
    virtual void setBackgroundChanged( void );

    protected:

    //! enable/disable transparency
    virtual void _setTransparent( const bool& value )
    {
      if( value == transparent_ ) return;
      transparent_ = value;
      setBackgroundChanged( true );
    }

    //! transparency
    virtual const bool& _transparent( void ) const
    { return transparent_; }

    //! tint
    virtual void _setTintColor( const QColor& );

    //! tint color
    virtual const QColor& _tintColor( void ) const
    { return tint_color_; }

    //! highlight
    virtual void _setHighlightColor( const QColor& );

    //! highlight color
    virtual const QColor& _highlightColor( void ) const
    { return highlight_color_; }

    //! highlighted
    virtual void _setHighlighted( bool value )
    { highlighted_ = value; }

    //! highlighted
    virtual const bool& _highlighted( void ) const
    { return highlighted_; }

    //! background pixmap
    virtual QPixmap& _backgroundPixmap( void )
    { return background_pixmap_; }

    //! background pixmap
    virtual const QPixmap& _backgroundPixmap( void ) const
    { return background_pixmap_; }

    //! opacity
    const double& _opacity( void ) const
    { return opacity_; }

    //!@name event handlers
    //@{

    //! move
    virtual void moveEvent( QMoveEvent* );

    //! resize
    virtual void resizeEvent( QResizeEvent* );

    //! show
    virtual void showEvent( QShowEvent* );

    //! enter event
    virtual void enterEvent( QEvent *event );

    //! leave event
    virtual void leaveEvent( QEvent *event );

    //! paint
    virtual void paintEvent( QPaintEvent* );

    //@}

    //! background changed
    const bool& _backgroundChanged( void ) const
    { return background_changed_; }

    //! paint background on devide
    virtual void _paintBackground( QPaintDevice&, const QRect& );

    //! paint main widget on devide
    /*! this must be re-implemented by derived classes */
    virtual void _paint( QPaintDevice&, const QRect& ) = 0;

    protected slots:

    //! reload background
    virtual void _reloadBackground( void );

    //! update background pixmap
    virtual void _updateBackgroundPixmap( void );

    private slots:

    //! update configuration
    void _updateConfiguration( void );

    private:

    //! actions
    void _installActions( void );

    //!@name actions
    //@{

    //! update background
    QAction *update_background_action_;

    //! reload background
    QAction* reload_background_action_;

    //@}

    //! transparency enabled
    bool transparent_;

    //! true when background needs to be reloaded
    bool background_changed_;

    //! tint color
    QColor tint_color_;

    //! true when pointer is in window
    bool highlighted_;

    //! window opacity
    double opacity_;

    //! tint color
    QColor highlight_color_;

    #ifdef Q_WS_WIN
    //! widget pixmap
    /*! it is used as widget storage when using full translucency */
    QPixmap widget_pixmap_;
    #endif

    //! background pixmap
    QPixmap background_pixmap_;

  };

};

#endif
