// $Id: ConfigurationDialog.cpp,v 1.26 2009/09/10 20:04:57 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file ConfigurationDialog.cc
  \brief configuration dialog
  \author Hugo Pereira
  \version $Revision: 1.26 $
  \date $Date: 2009/09/10 20:04:57 $
*/

#include <QButtonGroup>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>

#include "ConfigurationDialog.h"
#include "CustomToolBar.h"
#include "Debug.h"
#include "GridLayout.h"
#include "OptionCheckBox.h"
#include "OptionColorDisplay.h"
#include "OptionRadioButton.h"
#include "OptionSlider.h"
#include "OptionSpinBox.h"

#include "SvgConfiguration.h"
#include "TransparencyConfiguration.h"

using namespace std;

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
  BaseConfigurationDialog( parent )
{

  Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
  setWindowTitle( "Configuration - Taskbar" );

  // base configuration
  baseConfiguration( 0 );

  // transparency configuration
  QWidget* page;
  TRANSPARENCY::TransparencyConfiguration* transparency_configuration;

  page = &addPage( "Transparency", "Window transparency settings" );
  page->layout()->addWidget( transparency_configuration = new TRANSPARENCY::TransparencyConfiguration( page ));
  addOptionWidget( transparency_configuration );

  // generic widgets
  QGroupBox *box;
  GridLayout* grid_layout;
  OptionSpinBox* spinbox;
  OptionCheckBox* checkbox;
  OptionSlider* slider;

  // svg background
  page = &addPage( "Background", "Window background settings" );
  SVG::SvgConfiguration* svg_configuration( new SVG::SvgConfiguration( page ) );
  page->layout()->addWidget( svg_configuration);
  addOptionWidget( svg_configuration );

  // buttons
  page = &addPage( "Appearance", "Window appearance" );
  box = new QGroupBox( "Button size", page );
  grid_layout = new GridLayout();
  grid_layout->setSpacing(5);
  grid_layout->setMargin(5);
  grid_layout->setMaxCount(2);
  box->setLayout( grid_layout );
  page->layout()->addWidget( box );

  grid_layout->addWidget( new QLabel( "Margin: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "MARGIN" ) );
  spinbox->setToolTip( "Space between buttons and window border" );
  addOptionWidget( spinbox );

  grid_layout->addWidget( new QLabel( "Spacing: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "SPACING" ) );
  spinbox->setToolTip( "Space between consecutive buttons" );
  addOptionWidget( spinbox );

  grid_layout->addWidget( new QLabel( "Minimum size: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "MIN_BUTTON_SIZE" ) );
  spinbox->setToolTip( "Button minimum size (pixels)" );
  spinbox->setMinimum(0);
  spinbox->setMaximum(128);
  addOptionWidget( spinbox );

  grid_layout->addWidget( new QLabel( "Maximum size: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "MAX_BUTTON_SIZE" ) );
  spinbox->setToolTip( "Button maximum size (pixels)" );
  spinbox->setMinimum(0);
  spinbox->setMaximum(256);
  addOptionWidget( spinbox );

  // button highlight
  box = new QGroupBox( "Highlight", page );
  box->setLayout( new QVBoxLayout() );
  box->layout()->setMargin(5);
  box->layout()->setSpacing(5);
  page->layout()->addWidget( box );

  QHBoxLayout *h_layout = new QHBoxLayout();
  h_layout->setMargin(0);
  h_layout->setSpacing(5);
  box->layout()->addItem( h_layout );

  QButtonGroup* button_group( new QButtonGroup( this ) );
  button_group->setExclusive( true );

  OptionRadioButton* radio_button;
  h_layout->addWidget( radio_button = new OptionRadioButton( "none", box, "NO_HIGHLIGHT" ) );
  radio_button->setToolTip( "Do not highlight buttons" );
  addOptionWidget( radio_button );
  button_group->addButton( radio_button );

  h_layout->addWidget( radio_button = new OptionRadioButton( "On enter", box, "HIGHLIGHT_ON_ENTER" ) );
  radio_button->setToolTip( "Highlight buttons when hovered" );
  addOptionWidget( radio_button );
  button_group->addButton( radio_button );

  h_layout->addWidget( radio_button = new OptionRadioButton( "On click", box, "HIGHLIGHT_ON_CLICK" ) );
  radio_button->setToolTip( "Highlight buttons when clicked" );
  addOptionWidget( radio_button );
  button_group->addButton( radio_button );

  h_layout = new QHBoxLayout();
  h_layout->setMargin(0);
  h_layout->setSpacing(5);
  box->layout()->addItem( h_layout );

  h_layout->addWidget( new QLabel( "Intensity: ", box ) );
  h_layout->addWidget( slider = new OptionSlider( box, "HIGHLIGHT_INTENSITY" ) );
  slider->setScale( 100 );
  slider->slider().setMinimum( 0 );
  slider->slider().setMaximum( 100 );
  slider->setToolTip( "Button highlight intensity." );
  addOptionWidget( slider );

  // tooltips
  page = &addPage( "Misc", "Additional unsorted settings" );
  box = new QGroupBox( "Tooltips", page );
  box->setLayout( new QVBoxLayout() );
  box->layout()->setMargin(5);
  box->layout()->setSpacing(5);
  page->layout()->addWidget( box );

  grid_layout = new GridLayout();
  grid_layout->setSpacing(5);
  grid_layout->setMargin(0);
  grid_layout->setMaxCount(2);
  box->layout()->addItem( grid_layout );

  grid_layout->addWidget( new QLabel( "Tooltip size: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "TOOLTIP_HEIGHT" ) );
  spinbox->setMinimum(0);
  spinbox->setMaximum(1024);
  spinbox->setToolTip( "Tooltip window size (pixels)" );
  addOptionWidget( spinbox );

  grid_layout->addWidget( new QLabel( "Tooltip delay (miliseconds): ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "TOOLTIP_DELAY" ) );
  spinbox->setMinimum(0);
  spinbox->setMaximum(1024);
  spinbox->setToolTip( "Delay before the tooltip is shown" );
  addOptionWidget( spinbox );

  box->layout()->addWidget( checkbox = new OptionCheckBox( "Show tooltips", box, "SHOW_TOOLTIP" ) );
  checkbox->setToolTip( "Show tooltips" );
  addOptionWidget( checkbox );

  box->layout()->addWidget( checkbox = new OptionCheckBox( "Round tooltips", box, "ROUND_TOOLTIP" ) );
  checkbox->setToolTip( "Round tooltips" );
  addOptionWidget( checkbox );

  box->layout()->addWidget( checkbox = new OptionCheckBox( "Use transparency", box, "TRANSPARENT_TOOLTIP" ) );
  checkbox->setToolTip( "Transparent tooltips" );
  addOptionWidget( checkbox );

  // misc
  box = new QGroupBox( "Misc", page );
  grid_layout = new GridLayout();
  grid_layout->setSpacing(5);
  grid_layout->setMargin(5);
  grid_layout->setMaxCount(2);
  box->setLayout( grid_layout );
  page->layout()->addWidget( box );

  grid_layout->addWidget( new QLabel( "Displacement grid size: ", box ) );
  grid_layout->addWidget( spinbox = new OptionSpinBox( box, "GRID" ) );
  spinbox->setMinimum(1);
  spinbox->setToolTip( "Step size when moving the taskbar on desktop" );
  addOptionWidget( spinbox );

  // load initial configuration
  _read();

}
