
// $Id: PixmapList.cpp,v 1.11 2009/09/23 00:09:38 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file    PixmapList.cpp
  \brief   handles collection of pixmaps needed to draw a single button
  \author  Hugo Pereira
  \version $Revision: 1.11 $
  \date    $Date: 2009/09/23 00:09:38 $
*/

#include <QPainter>

// #include "Icons.h"
#include "PixmapEngine.h"
#include "PixmapList.h"
#include "SeparatorPixmap.h"
#include "XmlOptions.h"

using namespace std;

//_________________________________________________________________________
PixmapList::PixmapList(void):
  Counter( "PixmapList" ),
  max_size_( 0 ),
  min_size_( 0 ),
  size_increment_( 0 ),
  max_size_index_( 0 ),
  max_highlight_index_( 0 ),
  full_intensity_( 0 )
{}

//_________________________________________________________________________
void PixmapList::setAttributes( const TaskBarAttributes& bar_attributes, const TaskAttributes& attributes )
{

  Debug::Throw( "PixmapList::setAttributes.\n" );

  bar_attributes_ = bar_attributes;
  attributes_ = attributes;

  // size
  min_size_ = XmlOptions::get().get<unsigned int>( "MIN_BUTTON_SIZE" );
  max_size_ = XmlOptions::get().get<unsigned int>( "MAX_BUTTON_SIZE" );
  max_size_ = max( max_size_, min_size_ );
  max_size_index_ = max_size_ - min_size_ + 1;

  // if zoom is disabled, only store min and max size
  if( !bar_attributes.flag( TaskBarAttributes::ZOOM_ENABLED ) && bar_attributes.mode() != TaskBarAttributes::OSX )
  { max_size_index_ = min( max_size_index_, (unsigned int)2 ); }

  size_increment_ = max_size_index_ > 1 ? (max_size_-min_size_)/(max_size_index_-1):0;

  // see if highlighting is required
  bool highlight(
    (!attributes.flag( TaskAttributes::IS_SEPARATOR )) &&
    (XmlOptions::get().get<bool>("HIGHLIGHT_ON_ENTER") || XmlOptions::get().get<bool>("HIGHLIGHT_ON_CLICK") ) );

  // highlight intensity
  max_highlight_index_ = highlight ? XmlOptions::get().get<unsigned int>( "HIGHLIGHT_STEPS" ):1;

  // reset highlight properties
  full_intensity_ = XmlOptions::get().get<double>( "HIGHLIGHT_INTENSITY" );

  // allocate pixmap vector
  pixmaps_ = vector<QPixmap>( max_size_index_*max_highlight_index_ );

}

//_________________________________________________________________________
PixmapList::~PixmapList(void)
{}

//_________________________________________________________________________
const QPixmap& PixmapList::get( const unsigned int& size_index, const unsigned int& highlight_index ) const
{

  unsigned int index( _index( size_index, highlight_index ) );
  if( !pixmaps_[index].isNull() ) return pixmaps_[index];

  // see if base (un-highlighted) pixmap is already created
  // create it otherwise
  unsigned int base_index( _index( size_index, 0 ) );
  CustomPixmap base( pixmaps_[base_index] );
  if( base.isNull() )
  {

    // separator pixmap
    if( attributes_.flag( TaskAttributes::IS_SEPARATOR ) )
    {

      const TaskBarAttributes::Layout& layout( bar_attributes_.layout() );
      if( layout & TaskBarAttributes::HORIZONTAL )
      {

        base = SeparatorPixmap( size_index*size_increment_+4, bar_attributes_.flag( TaskBarAttributes::EXPAND_SEPARATORS ) ? (size_index*size_increment_ + min_size_):min_size_, layout );

      } else if( layout & TaskBarAttributes::VERTICAL ) {

        base = SeparatorPixmap( bar_attributes_.flag( TaskBarAttributes::EXPAND_SEPARATORS ) ? (size_index*size_increment_+min_size_):min_size_, size_index*size_increment_ + 4, layout );

      } else {

        base  = SeparatorPixmap( size_index*size_increment_ + 4, size_index*size_increment_ + 4, layout );

      }

    } else {

      // pixmap from icon
      const QPixmap& pixmap( PixmapEngine::get( attributes_.iconName() ) );
      assert( !pixmap.isNull() );

      //if( pixmap.isNull() ) pixmap = PixmapEngine::get( ICONS::DEFAULT );

      // scaled pixmap
      base = pixmap.scaled( QSize( size_index*size_increment_ + min_size_, size_index*size_increment_ + min_size_ ), Qt::KeepAspectRatio, Qt::SmoothTransformation );

    }

    // store as base (un-highlighted) pixmap
    if( base_index != index )
    { const_cast<List&>(pixmaps_)[base_index] = base; }

  }

  // highlight
  if( !attributes_.flag( TaskAttributes::IS_SEPARATOR ) && highlight_index && max_highlight_index_ > 1 )
  { base = base.highlighted( full_intensity_*(highlight_index+1)/( max_highlight_index_ )  ); }

  return ( const_cast<List&>(pixmaps_)[index] = base );

}

//_________________________________________________________________________
unsigned int PixmapList::_index( const unsigned int& size_index, const unsigned int& highlight_index ) const
{ return min( size_index, max_size_index_ - 1) + max_size_index_*min( highlight_index, max_highlight_index_-1 ); }
