#ifndef _TaskAttributes_h_
#define _TaskAttributes_h_

// $Id: TaskAttributes.h,v 1.21 2009/09/09 02:29:47 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file TaskAttributes.h
  \brief stores TaskAttributes attributesrmations
  \author Hugo Pereira
  \version $Revision: 1.21 $
  \date $Date: 2009/09/09 02:29:47 $
*/

#include <cstdlib>


#include <vector>
#include <QDomElement>
#include <QDomDocument>

#include "Counter.h"
#include "Options.h"
#include "TimeStamp.h"

//! single TaskAttributes attributesrmation
class TaskAttributes: public Counter
{

  public:

  //! task attributes list
  typedef std::vector<TaskAttributes> List;

  //! constructor
  TaskAttributes( void );

  //! constructor
  TaskAttributes( const QDomElement& element );

  //! dom element
  QDomElement domElement( QDomDocument& parent ) const;

  //! id
  const unsigned int& id() const
  { return id_; }

  //! equal to operator
  bool operator == (const TaskAttributes& attributes ) const
  {
    return
      id() == attributes.id()  &&
      name() == attributes.name() &&
      comments() == attributes.comments() &&
      command() == attributes.command() &&
      iconName() == attributes.iconName() &&
      flags() == attributes.flags();
  }

  //! less than operator
  bool operator < (const TaskAttributes& attributes ) const
  {
    if( id() != attributes.id() ) return id() < attributes.id();
    else if( name() != attributes.name() ) return name() < attributes.name();
    else if( comments() != attributes.comments() ) return comments() < attributes.comments();
    else if( command() != attributes.command() ) return command() < attributes.command();
    else if( iconName() != attributes.iconName() ) return iconName() < attributes.iconName();
    else if( flags() != attributes.flags() ) return flags() < attributes.flags();
    else return false;
  }

  //! return true if task attributes are different
  /*! unlike the == operator, it does not check on the ID */

  //! name
  const QString& name( void ) const
  { return name_; }

  //! name
  void setName( const QString& value )
  { name_ = value; }

  //! comments
  const QString& comments( void ) const
  { return comments_; }

  //! comments
  void setComments( const QString& buf )
  { comments_ = buf ;}

  //! command
  const QString& command( void ) const
  { return command_; }

  //! command
  void setCommand( const QString& value )
  { command_ = value; }

  //!@name icon name
  //@{

  //! load default icon filename
  QString defaultIconFilename( void ) const;

  //! icon
  const QString& iconName( void ) const
  { return icon_filename_; }

  //! icon
  void setIconName( const QString& value = QString() );

  //@}

  //! creation
  const TimeStamp& creation( void ) const
  { return creation_; }

  //! creation
  void setCreation( const TimeStamp& stamp = TimeStamp::now() )
  { creation_ = stamp; }

  //! modification
  const TimeStamp& modification( void ) const
  { return modification_; }

  //! modification
  void setModification( const TimeStamp& stamp = TimeStamp::now() )
  { modification_ = stamp; }

  //! last call
  const TimeStamp& lastCalled( void ) const
  { return last_called_; }

  //! last call
  void setLastCalled( const TimeStamp& stamp = TimeStamp::now() )
  { last_called_ = stamp; }

  //! execute command
  bool exec( void );

  //!@name flags
  //@{

  //! status in list
  enum Flag
  {
    NONE = 0,
    SHOW_NAME = 1<<0,
    IS_SEPARATOR = 1<<1
  };

  //! flags
  TaskAttributes& setFlag( const Flag& flag, const bool& value )
  {
    if( value ) flags_ |= flag;
    else flags_ &= (~flag);
    return *this;
  }

  //! flags
  void setFlags( const unsigned int& flags )
  { flags_ = flags; }

  //! flags
  bool flag( const Flag& flag ) const
  { return flags_ & flag; }

  //! flags
  const unsigned int& flags( void ) const
  { return flags_; }

  //! used to count separators
  class IsSeparatorFTor
  {

    public:

    //! predicate
    bool operator() (const TaskAttributes& attributes ) const
    { return attributes.flag( TaskAttributes::IS_SEPARATOR ); }

  };

  //@}

  private:

  //! id count
  static unsigned int& _idCount( void );

  //! id
  unsigned int id_;

  //! application name
  QString name_;

  //! comments
  QString comments_;

  //! application command
  QString command_;

  //! icon filename
  QString icon_filename_;

  //! flags
  unsigned int flags_;

  //!@name time stamps
  //@{

  //! creation
  TimeStamp creation_;

  //! modification
  TimeStamp modification_;

  //! last call
  TimeStamp last_called_;

  //@}

};

#endif
