
// $Id: TaskBarConfiguration.cpp,v 1.30 2009/09/09 02:29:48 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file TaskBarConfiguration.cc
  \brief edit task taskrmations
  \author Hugo Pereira
  \version $Revision: 1.30 $
  \date $Date: 2009/09/09 02:29:48 $
*/

#include <QLabel>
#include <QGroupBox>
#include <QLayout>
#include <QButtonGroup>

#include "GridLayout.h"
#include "AnimatedLineEditor.h"
#include "CustomSlider.h"
#include "TextEditor.h"
#include "Debug.h"
#include "XmlOptions.h"
#include "QtUtil.h"
#include "TaskBarConfiguration.h"

using namespace std;

//______________________________________________
TaskBarConfiguration::TaskBarConfiguration( QWidget* parent, const TaskBarAttributes& attributes ):
  QWidget( parent ),
  Counter( "TaskBarConfiguration" )
{
  Debug::Throw( "TaskBarConfiguration::TaskBarConfiguration.\n" );

  // layout
  setLayout( new QHBoxLayout() );
  layout()->setSpacing(10);
  layout()->setMargin(5);

  QVBoxLayout* v_layout = new QVBoxLayout();
  v_layout->setMargin(0);
  v_layout->setSpacing(5);
  layout()->addItem( v_layout );

  // name
  QGroupBox *box;
  GridLayout *grid_layout;
  v_layout->addWidget( box = new QGroupBox( "Name", this ));
  box->setLayout( new QVBoxLayout() );
  box->layout()->setSpacing(5);
  box->layout()->setMargin(5);

  box->layout()->addWidget( name_line_edit_ = new AnimatedLineEditor( box ) );
  name_line_edit_->setToolTip( "Taskbar name" );

//   // filename
//   string filename = string( "file name: <b>" ) + XmlOptions::get().raw("XML_FILE") + "</b>";
//   box->layout()->addWidget( new QLabel( filename.c_str(), box ) );

  // layout
  v_layout->addWidget( box = new QGroupBox( "Layout", this ) );
  grid_layout = new GridLayout();
  grid_layout->setSpacing(5);
  grid_layout->setMargin(5);
  grid_layout->setMaxCount(2);
  box->setLayout( grid_layout );

  grid_layout->addWidget( new QLabel( "Layout/alignment:", box ) );
  grid_layout->addWidget( layout_combobox_ = new QComboBox( box ) );

  const TaskBarAttributes::LayoutNameMap& layout_names( TaskBarAttributes::layoutNames() );
  for( TaskBarAttributes::LayoutNameMap::const_iterator iter = layout_names.begin(); iter != layout_names.end(); iter++ )
  { layout_combobox_->addItem( iter->second ); }

  layout_combobox_->setToolTip(
    "Choose taskbar layout and button \n"
    "alignment within taskbar." );

  connect( layout_combobox_, SIGNAL( activated( const QString& ) ), this, SLOT( _updateLayout( const QString& ) ) );

  // zoom
  grid_layout->addWidget( new QLabel( "Zoom mode:", box ) );
  grid_layout->addWidget( mode_combobox_ = new QComboBox( box ) );
  const TaskBarAttributes::ModeNameMap& mode_names( TaskBarAttributes::modeNames() );
  for( TaskBarAttributes::ModeNameMap::const_iterator iter = mode_names.begin(); iter != mode_names.end(); iter++ )
  { mode_combobox_->addItem( iter->second ); }
  mode_combobox_->setToolTip( "Taskbar zoom mode.\n" );

  connect( mode_combobox_, SIGNAL( activated( const QString& ) ), this, SLOT( _updateMode( const QString& ) ) );

  grid_layout->addWidget( new QLabel( "Number of columns: ", box ) );
  grid_layout->addWidget( column_spinbox_ = new QSpinBox( box ) );
  column_spinbox_->setMinimum( 2 );
  column_spinbox_->setMaximum( 20 );
  column_spinbox_->setToolTip( "Number of columns for the box layout" );

  grid_layout->addWidget( new QLabel( "Zoom scale: ", box ) );
  grid_layout->addWidget( scale_spinbox_ = new QSpinBox( box ) );
  scale_spinbox_->setMinimum( 1 );
  scale_spinbox_->setMaximum( 100 );
  scale_spinbox_->setToolTip( "zoom" );

  QCheckBox* checkbox;
  grid_layout->addWidget( checkbox = new QCheckBox( "zoom delay (milliseconds)", box ) );
  checkbox->setToolTip( "Enable/disable zoom delay.\n" );
  checkboxes_.insert( make_pair( TaskBarAttributes::ZOOM_ENABLED, checkbox ) );

  grid_layout->addWidget( delay_spinbox_ = new QSpinBox( box ) );
  delay_spinbox_->setMinimum( 0 );
  delay_spinbox_->setMaximum( 1000 );
  delay_spinbox_->setToolTip( "Interval between two updates while zooming" );

  // by default, delay spinbox is disabled, since the corresponding checkbox is not checked
  delay_spinbox_->setEnabled( false );

  connect( checkbox, SIGNAL( toggled( bool ) ), delay_spinbox_, SLOT( setEnabled( bool ) ) );

  // position box
  v_layout->addWidget( _positionBox( this ) );

  // flags
  layout()->addWidget( box = new QGroupBox( "Flags", this ) );
  v_layout = new QVBoxLayout();
  v_layout->setSpacing( 5 );
  v_layout->setMargin(5);
  box->setLayout( v_layout );

  v_layout->addWidget( checkbox = new QCheckBox( "Lock position", box ) );
  checkbox->setToolTip(
    "Lock taskbar position on screen.\n"
    "Note: this option overrides the\n "
    "global configuration settings.\n" );
  checkboxes_.insert( make_pair( TaskBarAttributes::LOCK_POSITION, checkbox ) );

  v_layout->addWidget( checkbox = new QCheckBox( "Auto hide", box ) );
  checkbox->setToolTip( "Hide taskbar when unused.\n" );
  checkboxes_.insert( make_pair( TaskBarAttributes::AUTO_HIDE, checkbox ) );

  v_layout->addWidget( checkbox = new QCheckBox( "Stays on top", box ) );
  checkbox->setToolTip( "Controls if taskbar must stay on top of all windows\n");
  checkboxes_.insert( make_pair( TaskBarAttributes::STAYS_ON_TOP, checkbox ) );

  v_layout->addWidget( checkbox =  new QCheckBox( "Sticky", box ) );
  checkbox->setToolTip( "Make window appear on all desktops" );
  checkboxes_.insert( make_pair( TaskBarAttributes::STICKY, checkbox ) );

  v_layout->addWidget( checkbox =  new QCheckBox( "Hide from taskbar", box ) );
  checkbox->setToolTip( "Hide window from taskbar" );
  checkboxes_.insert( make_pair( TaskBarAttributes::HIDE_FROM_TASKBAR, checkbox ) );

  v_layout->addWidget( checkbox = new QCheckBox( "Round corners", box ) );
  checkbox->setToolTip( "Round taskbar corners\n");
  checkboxes_.insert( make_pair( TaskBarAttributes::ROUND_CORNERS, checkbox ) );

  QFrame* frame;
  v_layout->addWidget( frame = new QFrame( this ) );
  frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // button flags
  v_layout->addWidget( checkbox = new QCheckBox( "Lock buttons position", box ) );
  checkbox->setToolTip(
      "Lock buttons position in taskbar.\n"
      "Note: this option overrides the\n "
      "global configuration settings.\n" );
  checkboxes_.insert( make_pair( TaskBarAttributes::LOCK_BUTTONS, checkbox ) );

  v_layout->addWidget( checkbox = new QCheckBox( "Show button names", box ) );
  checkbox->setToolTip(
    "Show button names in taskbar.\n"
    "Note: this option overrides the\n "
    "global configuration settings.\n"
    "It gets overriden by individual\n"
    "button settings" );
  checkboxes_.insert( make_pair( TaskBarAttributes::SHOW_NAME, checkbox ) );

  v_layout->addWidget( checkbox = new QCheckBox( "expand separators", box ) );
  checkbox->setToolTip(
      "Controls if separators are expanded\n"
      "while zooming in OsX mode\n "
      ".\n" );
  checkboxes_.insert( make_pair( TaskBarAttributes::EXPAND_SEPARATORS, checkbox ) );

  for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); iter++ )
  { connect( iter->second, SIGNAL( toggled( bool ) ), SLOT( _updateCheckBoxes() ) ); }

  v_layout->addStretch(1);

  setAttributes( attributes );

}

//_______________________________________
void TaskBarConfiguration::setAttributes( const TaskBarAttributes& attributes )
{
  Debug::Throw( "TaskBarConfiguration::SetAttributes.\n" );

  attributes_ = attributes;
  backup_attributes_ = attributes;

  name_line_edit_->setText( attributes.name() );
  layout_combobox_->setCurrentIndex( layout_combobox_->findText( attributes.layoutName() ) );
  mode_combobox_->setCurrentIndex( mode_combobox_->findText( attributes.modeName() ) );
  delay_spinbox_->setValue( attributes.zoomDelay() );
  column_spinbox_->setValue( attributes.numColumns() );
  scale_spinbox_->setValue( int( attributes.scale()*10 ) );

  for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); iter++ )
  { iter->second->setChecked( attributes.flag( iter->first ) ); }

  if( attributes.positionMode() != TaskBarAttributes::ANY_POSITION )
  {
    for( ButtonMap::iterator iter = position_buttons_.begin(); iter != position_buttons_.end(); iter++ )
    { if( iter->second == attributes.positionMode() ) iter->first->setChecked( true ); }
  }

  _updateLayout( attributes.layoutName() );
  _updateMode( attributes.modeName() );

}
//_______________________________________
const TaskBarAttributes& TaskBarConfiguration::attributes( void )
{

  Debug::Throw("TaskBarConfiguration::Attributes.\n");

  attributes_.setName( name_line_edit_->text() );
  attributes_.setLayout( layout_combobox_->currentText() );
  attributes_.setMode( mode_combobox_->currentText() );
  attributes_.setZoomDelay( delay_spinbox_->value() );
  attributes_.setNumColumns( column_spinbox_->value() );
  attributes_.setScale( double(scale_spinbox_->value())/10 );

  for( CheckBoxMap::iterator iter = checkboxes_.begin(); iter != checkboxes_.end(); iter++ )
  { attributes_.setFlag( iter->first, iter->second->isChecked() ); }

  // set prefered position
  unsigned int position( TaskBarAttributes::ANY_POSITION );
  for( ButtonMap::iterator iter = position_buttons_.begin(); iter != position_buttons_.end(); iter++ )
  { if( iter->first->isChecked() )  position = iter->second; }

  attributes_.setPositionMode( position );

  // check for incompatibilities in switches
  return attributes_;

}

//_______________________________________________
void TaskBarConfiguration::_togglePosition( QAbstractButton* button )
{

  Debug::Throw( "TaskBarConfiguration::_togglePosition" );
  ButtonMap::iterator clicked_iter( position_buttons_.find( button ) );
  assert( clicked_iter != position_buttons_.end() );
  if( !clicked_iter->first->isChecked() ) return;

  for( ButtonMap::iterator iter = position_buttons_.begin(); iter != position_buttons_.end(); iter++ )
  { if( iter->first != clicked_iter->first ) iter->first->setChecked( false ); }

  // change checkboxes enability
  _updateCheckBoxes();

}

//_______________________________________________
void TaskBarConfiguration::_updateLayout( const QString& value )
{
  column_spinbox_->setEnabled( value == TaskBarAttributes::layoutName( TaskBarAttributes::BOX ) );
  _updateCheckBoxes();
}

//_______________________________________________
void TaskBarConfiguration::_updateMode( const QString& value )
{ scale_spinbox_->setEnabled( value == TaskBarAttributes::modeName( TaskBarAttributes::OSX ) ); }

//_______________________________________________
QWidget* TaskBarConfiguration::_positionBox( QWidget* parent )
{
  Debug::Throw( "TaskBarConfiguration::_positionBox.\n" );
  QGroupBox* box = new QGroupBox( "Prefered position", parent );
  box->setLayout( new QVBoxLayout() );
  box->layout()->setSpacing(5);
  box->layout()->setMargin(5);

  QGridLayout *grid_layout = new QGridLayout();
  grid_layout->setSpacing(10);
  grid_layout->setMargin(0);
  box->layout()->addItem( grid_layout );

  // needed to match button to position mode
  static TaskBarAttributes::Position positions[3] =
  {
    TaskBarAttributes::LEFT_POSITION,
    TaskBarAttributes::CENTER_POSITION,
    TaskBarAttributes::RIGHT_POSITION
  };

  QButtonGroup* button_group =  new QButtonGroup( box );
  button_group->setExclusive( false );
  QPushButton *button;
  for( unsigned int i=0; i<3; i++ )
  {
    // top buttons
    unsigned int position = TaskBarAttributes::TOP_POSITION | positions[i];
    button_group->addButton( button = new QPushButton( box ) );
    button->setToolTip( TaskBarAttributes::positionName( position ) );
    grid_layout->addWidget( button, 0, i );
    position_buttons_.insert( make_pair( button, position ) );

    // bottom buttons
    position = TaskBarAttributes::BOTTOM_POSITION | positions[i];
    button_group->addButton( button = new QPushButton( box ) );
    button->setToolTip( TaskBarAttributes::positionName( position ) );
    grid_layout->addWidget( button, 2, i );
    position_buttons_.insert( make_pair( button, position  ) );

  }

  // left centered
  button_group->addButton( button = new QPushButton( box ) );
  button->setToolTip( TaskBarAttributes::positionName( TaskBarAttributes::LEFT_CENTER_POSITION ) );
  grid_layout->addWidget( button, 1, 0 );
  position_buttons_.insert( make_pair( button, TaskBarAttributes::LEFT_CENTER_POSITION ) );

  // right centered
  button_group->addButton( button = new QPushButton( box ) );
  button->setToolTip( TaskBarAttributes::positionName( TaskBarAttributes::RIGHT_CENTER_POSITION ) );
  grid_layout->addWidget( button, 1, 2 );
  position_buttons_.insert( make_pair( button, TaskBarAttributes::RIGHT_CENTER_POSITION ) );

  // set size and toggle
  for( ButtonMap::iterator iter = position_buttons_.begin(); iter != position_buttons_.end(); iter++ )
  { iter->first->setCheckable( true ); }

  // set signal
  connect( button_group, SIGNAL( buttonClicked( QAbstractButton* ) ), SLOT( _togglePosition( QAbstractButton* ) ) );
  return box;

}

//_______________________________________________
void TaskBarConfiguration::_updateCheckBoxes( void )
{
{ Debug::Throw("TaskBarConfiguration::_updateCheckBoxes.\n"); }

  attributes();

  // AUTO_HIDE is enabled only if and layout match with position.
  checkboxes_[TaskBarAttributes::AUTO_HIDE ]->setEnabled(
    ( ( attributes_.positionMode() == TaskBarAttributes::BOTTOM_CENTER_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::BOTTOM_LEFT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::BOTTOM_RIGHT_POSITION ) &&
    attributes_.layout() ==  TaskBarAttributes::HORIZONTAL_BOTTOM ) ||

    ( ( attributes_.positionMode() == TaskBarAttributes::TOP_CENTER_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::TOP_LEFT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::TOP_RIGHT_POSITION ) &&
    attributes_.layout() ==  TaskBarAttributes::HORIZONTAL_TOP ) ||

    ( ( attributes_.positionMode() == TaskBarAttributes::TOP_LEFT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::BOTTOM_LEFT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::LEFT_CENTER_POSITION ) &&
    attributes_.layout() ==  TaskBarAttributes::VERTICAL_LEFT ) ||

    ( ( attributes_.positionMode() == TaskBarAttributes::TOP_RIGHT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::BOTTOM_RIGHT_POSITION ||
    attributes_.positionMode() == TaskBarAttributes::RIGHT_CENTER_POSITION ) &&
    attributes_.layout() ==  TaskBarAttributes::VERTICAL_RIGHT ) );


}
