// $Id: TaskBarMinimum.cpp,v 1.68 2009/09/09 02:29:48 hpereira Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is_ distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file    TaskBarMinimum.cc
  \brief   transparent frame to display TaskButton(s)
  \author  Hugo Pereira
  \version $Revision: 1.68 $
  \date    $Date: 2009/09/09 02:29:48 $
*/

#include <QAction>
#include <QApplication>
#include <QDesktopWidget>

#include "RoundedRegion.h"
#include "SvgEngine.h"
#include "TaskBarMinimum.h"
#include "TaskToolTip.h"
#include "XmlOptions.h"

#include "CompositeEngine.h"

using namespace std;

//____________________________________________________
TaskBarMinimum::TaskBarMinimum( QWidget* parent ):
  TaskBarBase( parent ),
  root_pixmap_changed_( true )
{ Debug::Throw( "TaskBarMinimum::TaskBarMinimum.\n" ); }

//____________________________________________________
void TaskBarMinimum::_updateRootPixmap( void )
{

  Debug::Throw( "TaskBarMinimum::_updateRootPixmap" );
  _setRootPixmapChanged( false );
  if( Debug::level() >= 1 )
  {
    // solid background
    _setRootPixmap( QPixmap( size() ) );
    _rootPixmap().fill( palette().color( backgroundRole() ) );

  } else if( TRANSPARENCY::CompositeEngine::get().isEnabled() ) {

    _setRootPixmap( QPixmap( size() ) );
    QPainter painter( &_rootPixmap() );
    painter.setRenderHints(QPainter::SmoothPixmapTransform);
    painter.setCompositionMode(QPainter::CompositionMode_Source );
    painter.fillRect( _rootPixmap().rect(), Qt::transparent);

  } else {

    _setRootPixmap( QPixmap::grabWindow(
      QApplication::desktop()->winId(),
      geometry().x(), geometry().y(),
      geometry().width(), geometry().height() ) );

  }
}

//___________________________________________________
void TaskBarMinimum::_updateBackgroundPixmap( void )
{
  Debug::Throw( "TaskBarMinimum::TaskBarFixed.\n" );
  if( _rootPixmapChanged() )
  {
    // there is no need to update the root pixmap when composite engine is off,
    // unless during resize events, for which the method is called directly
    if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _updateRootPixmap();
    else _setRootPixmapChanged( false );
  }

  TaskBarBase::_updateBackgroundPixmap();

  // need to draw svg on background pixmap if button list is empty
  // due to the way empty taskbars are implemented
  if( _buttons().empty() &&_useSvg() && SVG::SvgEngine::get().isValid() )
  {
    QPainter painter( &_backgroundPixmap() );
    painter.setRenderHints(QPainter::SmoothPixmapTransform);
    painter.setCompositionMode(QPainter::CompositionMode_Source );
    painter.drawPixmap( QPoint(0,0),  SVG::SvgEngine::get().get( size() ) );
  }

}

//______________________________________________
void TaskBarMinimum::resizeEvent( QResizeEvent* event )
{
  _updateRootPixmap();
  TaskBarBase::resizeEvent( event );
}

//______________________________________________________
void TaskBarMinimum::_resize( void )
{

  Debug::Throw( "TaskBarMinimum::_resize.\n" );

  // if no tasks, set empty message and exit
  if( _buttons().empty() ) {
    resize( emptySize() );
    return;
  }

  // dimensions
  _setDimensions();
  _updateMinimumSize();
  _updateMaximumSize();
  _updateMinimumRectPosition();

  // svg
  if( _useSvg() && SVG::SvgEngine::get().isValid() ) _preloadSvg();

  // make sure taskbar is in minimum state
  // resize
  _setRootPixmapChanged( true );
  _setAction( MINIMUM );
  resize( _maximumSize() );

}

//___________________________________________________
void TaskBarMinimum::_placeButtons( void )
{

  Debug::Throw( "TaskBarMinimum::_placeButtons.\n" );

  // check that there is at least one button
  if( _buttons().empty() ) return;

  // starting positions
  int x(0);
  int y(0);

  const TaskBarAttributes::Layout &layout( attributes().layout() );

  if( layout & ( TaskBarAttributes::VERTICAL | TaskBarAttributes::HORIZONTAL ) )
  {

    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {

      (*iter)->setRect( QPoint(x,y), QSize( (*iter)->currentSize().width() + _spacing(), (*iter)->currentSize().height() + _spacing()  ) );

      // update to next button
      if( layout & TaskBarAttributes::VERTICAL ) y+= (*iter)->rect().height();
      else if( layout & TaskBarAttributes::HORIZONTAL )  x += (*iter)->rect().width();
    }

  } else if( layout & TaskBarAttributes::BOX ) {

    unsigned int column_index(0);
    unsigned int line_index(0);

    // compute column width/line height
    vector<unsigned int> column_width( _numColumns(), 0 );
    vector<unsigned int> line_height( _numLines(), 0 );
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {

      line_height[line_index] = max( line_height[line_index], (*iter)->currentSize().height() + _spacing() );
      column_width[column_index] = max( column_width[column_index], (*iter)->currentSize().width() + _spacing() );
      column_index++;
      if( column_index >= _numColumns() ) {
        column_index = 0;
        line_index++;
      }

    }

    // place buttons
    column_index = 0;
    line_index = 0;
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {
      (*iter)->setRect( QPoint( x, y ), QSize( column_width[column_index], line_height[line_index] ) );
      x+=column_width[column_index];
      column_index++;
      if( column_index >= _numColumns() ) {
        x = 0;
        column_index = 0;

        y+= line_height[line_index];
        line_index++;
      }
    }
  }

}

//______________________________________________________
void TaskBarMinimum::_addOffsets( void )
{

  // dump
  Debug::Throw() << "TaskBarMinimum::_addOffsets -"
    << " size: (" << width() << "," << height() << ")"
    << endl;

  // get bounding rect for current button sizes
  QRect bounding_rect;
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    bounding_rect.setWidth( max( bounding_rect.width(), (*iter)->rect().right() ) );
    bounding_rect.setHeight( max( bounding_rect.height(), (*iter)->rect().bottom() ) );
  }

  // compute offsets
  QPoint offset( (size().width() - bounding_rect.width())/2, (size().height() - bounding_rect.height())/2 );
  const TaskBarAttributes::Layout &layout( attributes().layout() );

  // add offsets
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    QPoint button_offset( offset );
    if( layout & TaskBarAttributes::VERTICAL )
    {

      if( layout & TaskBarAttributes::TOP_LEFT ) button_offset.setX(_margin());
      else if( layout & TaskBarAttributes::BOTTOM_RIGHT ) button_offset.setX( width() - (*iter)->rect().width() - _margin() );
      else if( layout & TaskBarAttributes::CENTER ) button_offset.setX( (width() - (*iter)->rect().width())/2 );

    } else if( layout & TaskBarAttributes::HORIZONTAL ) {

      if( layout & TaskBarAttributes::TOP_LEFT ) button_offset.setY(_margin());
      else if( layout & TaskBarAttributes::BOTTOM_RIGHT ) button_offset.setY( height() - (*iter)->rect().height() - _margin() );
      else if( layout & TaskBarAttributes::CENTER ) button_offset.setY( (height() - (*iter)->rect().height())/2 );

    }

    (*iter)->setRect( (*iter)->rect().translated( button_offset ) );

  }

}

//___________________________________________________
bool TaskBarMinimum::_drawButtons( QPaintDevice& device, QRect rect )
{

  // if no tasks, do nothing
  if( _buttons().empty() ) return false;

  // place the buttons
  _placeButtons();
  _addOffsets();

  // redraw
  QPainter painter( &device );
  painter.setClipRect( rect );

  if( TRANSPARENCY::CompositeEngine::get().isEnabled() )
  {
    painter.setRenderHints(QPainter::SmoothPixmapTransform);
    painter.setCompositionMode(QPainter::CompositionMode_Source );
  }

  // draw root pixmap into entire window
  painter.drawPixmap( 0, 0, _rootPixmap() );

  // draw background pixmap clipped to visibleRect
  _updateVisibleRect();
  _drawVisibleRect( painter );

  // draw buttons, translated by offset if needed
  painter.setCompositionMode(QPainter::CompositionMode_SourceOver );
  painter.translate( _offset() );

  bool out( false );
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {

    (*iter)->paint( painter );
    if( _action() != ENTER )
    {
      (*iter)->updateDirection();
      (*iter)->updateHighlightDirection();
      out |= ((*iter)->direction() != TaskButton::NONE);
      out |= (*iter)->highlightDirection() != TaskButton::NOHIGHLIGHT;
    }

  }

  painter.end();

  return out;

}

//___________________________________________________
void TaskBarMinimum::_drawVisibleRect( QPainter& painter )
{
  painter.save();

  if( _useSvg() && SVG::SvgEngine::get().isValid() )
  {

    painter.translate( _visibleRect().topLeft() );
    painter.drawPixmap( QPoint(0,0), SVG::SvgEngine::get().get( _visibleRect().size() ) );

  } else {
    if( attributes().flag( TaskBarAttributes::ROUND_CORNERS ) ) painter.setClipRegion( RoundedRegion( _visibleRect(), RoundedRegion::ALL ) );
    else painter.setClipRegion( _visibleRect() );
    painter.drawPixmap( 0, 0, _backgroundPixmap() );
  }
  painter.restore();
}

//_____________________________________________________________
void TaskBarMinimum::_updateMinimumSize( void )
{

  Debug::Throw( "TaskBarMinimum::_updateMinimumSize.\n" );

  // compute minimum size
  QSize minimum_size;

  // compute visible rect
  const TaskBarAttributes::Layout& layout ( attributes().layout() );
  if( layout & TaskBarAttributes::VERTICAL )
  {
    for( TaskButton::List::const_iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {
      minimum_size.setHeight( minimum_size.height() + (*iter)->minimumSize().height() + _spacing() );
      minimum_size.setWidth( max<int>( minimum_size.width(), (*iter)->minimumSize().width() + _spacing() + 2*_margin() ) );
    }

    minimum_size.setHeight( minimum_size.height() + 2*_margin() );

  } else if( layout & TaskBarAttributes::HORIZONTAL ) {

    for( TaskButton::List::const_iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {
      minimum_size.setWidth( minimum_size.width() + (*iter)->minimumSize().width() + _spacing() );
      minimum_size.setHeight( max<int>( minimum_size.height(), (*iter)->minimumSize().height() + _spacing() + 2*_margin() ) );
    }

    minimum_size.setWidth( minimum_size.width() + 2*_margin() );

  } else {

    unsigned int column_index(0);
    unsigned int line_index(0);

    // compute column width/line height
    vector<unsigned int> column_width( _numColumns(), 0 );
    vector<unsigned int> line_height( _numLines(), 0 );
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {

      line_height[line_index] = max( line_height[line_index], (*iter)->currentSize().height() + _spacing() );
      column_width[column_index] = max( column_width[column_index], (*iter)->currentSize().width() + _spacing() );
      column_index++;
      if( column_index >= _numColumns() ) {
        column_index = 0;
        line_index++;
      }

    }

    int line_width(0);
    column_index = 0;
    line_index = 0;
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {

      line_width += column_width[column_index];
      column_index++;

      // change column
      if( column_index >= _numColumns() )
      {
        minimum_size.setWidth( max( minimum_size.width(), line_width ) );
        minimum_size.setHeight( minimum_size.height()+line_height[line_index] );

        line_width = 0;
        column_index = 0;
        line_index++;

      }

    }

    minimum_size.setHeight( minimum_size.height() + line_height.back() + 2*_margin() );
    minimum_size.setWidth( minimum_size.width() + 2*_margin() );

  }

  // update
  _setMinimumSize( minimum_size );
  return;

}

//_____________________________________________________________
void TaskBarMinimum::_updateMaximumSize( void )
{

  Debug::Throw( "TaskBarMinimum::_updateMaximumSize.\n" );

  // compute maximum size
  const QSize& minimum_size( _minimumSize() );
  QSize maximum_size( _minimumSize() );
  for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
  {
    maximum_size.setWidth( max( maximum_size.width(), minimum_size.width() + (*iter)->maximumSize().width() - (*iter)->minimumSize().width() ) );
    maximum_size.setHeight( max( maximum_size.height(), minimum_size.height() + (*iter)->maximumSize().height() - (*iter)->minimumSize().height() ) );
  }

  // update
  _setMaximumSize( maximum_size );
  return;

}

//_____________________________________________________________
void TaskBarMinimum::_updateMinimumRectPosition( void )
{

  Debug::Throw( "TaskBarMinimum::_updateMinimumRectPosition.\n" );

  QRect minimum_rect( _minimumRect() );
  QPoint offset;
  const TaskBarAttributes::Layout& layout ( attributes().layout() );
  if( layout & TaskBarAttributes::VERTICAL ) {

    offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );
    if( layout & TaskBarAttributes::BOTTOM_RIGHT ) offset.setX( _maximumSize().width() - minimum_rect.width() );
    else if( layout & TaskBarAttributes::CENTER ) offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );

  } else if( layout & TaskBarAttributes::HORIZONTAL ) {
    offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );
    if( layout & TaskBarAttributes::BOTTOM_RIGHT ) offset.setY( _maximumSize().height() - minimum_rect.height() );
    else if( layout & TaskBarAttributes::CENTER ) offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );

  } else  {

    offset.setX( ( _maximumSize().width() - minimum_rect.width() )/2 );
    offset.setY( ( _maximumSize().height() - minimum_rect.height() )/2 );

  }

  minimum_rect.moveTo( offset );
  _setMinimumRect( minimum_rect );

}

//_____________________________________________________________
void TaskBarMinimum::_updateVisibleRect( void )
{

  Debug::Throw( "TaskBarMinimum::_updateVisibleRect.\n" );

  QRect visible_rect;
  const TaskBarAttributes::Layout& layout ( attributes().layout() );
  if( layout & TaskBarAttributes::VERTICAL ) {

    // width is from minimum size
    visible_rect.setWidth( _minimumSize().width() );

    // get bounding height and y offset
    bool first( true );
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {
      visible_rect.setTop( first ? (*iter)->rect().top() : min( visible_rect.top(), (*iter)->rect().top() ) );
      visible_rect.setBottom( max( visible_rect.bottom(), (*iter)->rect().bottom() ) );
      first = false;
    }

    visible_rect.setTop( visible_rect.top() - _margin() );
    visible_rect.setBottom( visible_rect.bottom() + _margin() );

    if( layout & TaskBarAttributes::BOTTOM_RIGHT ) visible_rect.moveLeft(  _maximumSize().width() - visible_rect.width() );
    else if( layout & TaskBarAttributes::CENTER ) visible_rect.moveLeft( ( _maximumSize().width() - visible_rect.width() )/2 );

  } else if( layout & TaskBarAttributes::HORIZONTAL ) {

    // height is from minimum size
    visible_rect.setHeight( _minimumSize().height() );

    // get bounding height and y offset
    bool first( true );
    for( TaskButton::List::iterator iter = _buttons().begin(); iter != _buttons().end(); iter++ )
    {
      visible_rect.setLeft( first ? (*iter)->rect().left() : min( visible_rect.left(), (*iter)->rect().left() ) );
      visible_rect.setRight( max( visible_rect.right(), (*iter)->rect().right() ) );
      first = false;
    }

    visible_rect.setLeft( visible_rect.left() - _margin() );
    visible_rect.setRight( visible_rect.right() + _margin() );

    if( layout & TaskBarAttributes::BOTTOM_RIGHT ) visible_rect.moveTop(  _maximumSize().height() - visible_rect.height() );
    else if( layout & TaskBarAttributes::CENTER ) visible_rect.moveTop( ( _maximumSize().height() - visible_rect.height() )/2 );

  } else {

    visible_rect.setSize( _minimumSize() );
    visible_rect.moveTo( ( _maximumSize().width() - visible_rect.width() )/2, ( _maximumSize().height() - visible_rect.height() )/2 );

  }

  // add offset (for auto-hide)
  visible_rect.translate( _offset() );

  // in case of autoHide, one make sure there is at least
  // a thin band drawn at the bottom, that match the hidden geometry
  if( attributes().flag( TaskBarAttributes::AUTO_HIDE) )
  {

    const TaskBarAttributes::Layout& layout ( attributes().layout() );
    if( layout & TaskBarAttributes::HORIZONTAL )
    {
      if( layout & TaskBarAttributes::TOP_LEFT )  visible_rect.setBottom( max(  visible_rect.bottom(), 3 ) );
      else visible_rect.setTop( min( visible_rect.top(), height()-3 ) );
    } else {
      if( layout & TaskBarAttributes::TOP_LEFT ) visible_rect.setRight( max( visible_rect.right(), 3 ) );
      else visible_rect.setLeft( min( visible_rect.left(), width()-3 ) );
    }

  }

  _setVisibleRect( visible_rect );

  return;

}

//_____________________________________________________________
bool TaskBarMinimum::_updateGeometry( Action action )
{

  Debug::Throw() << "TaskBarMinimum::_updateGeometry - action: " << action << endl;

  if( action == NONE ) return false;

  // minimum action
  if( action == MINIMUM )
  {

    setMask( _minimumRect() );
    if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _setRootPixmapChanged( true );
    _setAction( NONE );
    return false;

  }

  // enter action
  if( action == ENTER ) {

    if( _rootPixmapChanged() )
    {
      // there is no need to update the root pixmap when composite engine is off,
      // unless during resize events, for which the method is called directly
      if( !TRANSPARENCY::CompositeEngine::get().isEnabled() ) _updateRootPixmap();
      else _setRootPixmapChanged( false );
    }

    if( !TaskBarBase::_updateGeometry( action ) )
    {

      _setAction( NONE );
      setMask( rect() );
      return false;

    } else return true;

  }

  // leave action
  if( action == LEAVE ) {

    if( !_rootPixmapChanged() )
    {
      setMask( _minimumRect() );
      _setRootPixmapChanged( true );
    }

    if( !TaskBarBase::_updateGeometry( action ) )
    {

      _setAction( NONE );
      return false;

    } else return true;

  }

  return false;

}

//_________________________________________________________________
void TaskBarMinimum::_preloadSvg( void ) const
{

  Debug::Throw( "TaskBarMinimum::_preloadSvg.\n" );

  assert( _useSvg() );

  // loop over all possible sizes to preload cache
  QSize maximum_size( maximum_size_ );
  QSize minimum_size( minimum_rect_.size() );

  SVG::SvgId::List ids;
  const TaskBarAttributes::Layout& layout( attributes().layout() );
  if( layout & ( TaskBarAttributes::HORIZONTAL ) )
  {

    for( int width = minimum_size.width(); width <= maximum_size.width(); width++ )
    { ids.push_back( QSize( width, minimum_size.height() ) ); }

  } else if( layout & (TaskBarAttributes::HORIZONTAL ) ) {

    for( int height = minimum_size.height(); height <= maximum_size.height(); height++ )
    { ids.push_back( QSize( minimum_size.width(), height ) ); }

  } else ids.push_back( minimum_size );

  // load sizes
  SVG::SvgEngine::get().preload( ids );

  return;

}
