
// $Id: TaskButton.cpp,v 1.31 2009/09/09 02:29:48 hpereira Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file    TaskButton.cc
  \brief   transparent label to display task icon
  \author  Hugo Pereira
  \version $Revision: 1.31 $
  \date    $Date: 2009/09/09 02:29:48 $
*/
#include <QApplication>
#include <QFont>
#include <cmath>

#include "Debug.h"
#include "Effects.h"
#include "Options.h"
#include "SeparatorPixmap.h"
#include "TaskButton.h"
#include "XmlOptions.h"

using namespace std;

// this class debug level
static const int debug_level = 1;

//_____________________________________________
void TaskButton::setAttributes( const TaskBarAttributes& bar_attributes, const TaskAttributes& attributes )
{

  Debug::Throw() << "TaskButton::SetAttributes - " << attributes.name() << endl;

  // store initial max size (for the need_resize flag)
  QSize old_size( maximumSize() );

  // store task
  attributes_ = attributes;

  // highlight steps
  maximum_highlight_index_ = XmlOptions::get().get<unsigned int>( "HIGHLIGHT_STEPS" ) + 1;

  // pixmap max/max size
  unsigned int max_size( XmlOptions::get().get<unsigned int>( "MAX_BUTTON_SIZE" ) );

  //! clear pixmap list
  _pixmaps().setAttributes( bar_attributes, attributes );

  // for separators, only create one pixmap
  if( attributes.flag( TaskAttributes::IS_SEPARATOR ) ) show_name_ = false;
  else {

    // add needed space for text
    unsigned int width = max_size;
    unsigned int height = max_size;
    show_name_ = bar_attributes.flag( TaskBarAttributes::SHOW_NAME ) || attributes_.flag( TaskAttributes::SHOW_NAME );
    if( show_name_ )
    {

      // retrieve options
      shadow_color_ = QColor( XmlOptions::get().raw( "TRANSPARENCY_SHADOW_COLOR" ) );
      shadow_offset_ = XmlOptions::get().get<int>( "TRANSPARENCY_SHADOW_OFFSET" );
      foreground_color_ = QColor( XmlOptions::get().raw( "TRANSPARENCY_FOREGROUND_COLOR" ) );

      QFontMetrics font( qApp->fontMetrics() );
      text_size_ = font.size( 0, attributes_.name() );
      width = max<int>( width, text_size_.width() );
      height += text_size_.height();

    }
  }

  // validity
  /* button must be a separator, or its pixmap must be valid, or its name must be valid and shown */
  _setIsValid(
    attributes.flag( TaskAttributes::IS_SEPARATOR ) ||
    !CustomPixmap( attributes.iconName() ).isNull() ||
    ( ( bar_attributes.flag( TaskBarAttributes::SHOW_NAME ) ||
    attributes_.flag( TaskAttributes::SHOW_NAME ) ) &&
    !attributes.name().isEmpty() ) );

  // set button maximum size
  _setNeedResize( old_size != maximumSize() );
  return;
}

//___________________________________________________
QSize TaskButton::size( const unsigned int& index) const
{

  // check pixmaps
  if( !_pixmaps().maxSizeIndex() ) return QSize();

  // size
  if( attributes_.flag( TaskAttributes::IS_SEPARATOR ) ) return _pixmaps().get( index, 0 ).size();

  // return size (adding text size if needed)
  return QSize(
    std::max( text_size_.width(), _pixmaps().get( index, 0 ).width() ),
    text_size_.height()+ _pixmaps().get( index, 0 ).height() );

}

//__________________________________________________________
void TaskButton::setDirection( const TaskButton::Direction& value )
{
  Debug::Throw( "TaskButton::setDirection\n" );
  if( value == direction_ ) return;

  // update value
  direction_ = value;

}

//__________________________________________________________
const TaskButton::HighlightDirection& TaskButton::updateHighlightDirection( void )
{

  if( highlight_direction_ == LIGHTER )
  {

    if( highlight_index_ < maximum_highlight_index_-1 )
    {

      highlight_index_+=0.1;
      _updateTrueHighlightIndex();

    } else highlight_direction_ = DARKER;

  } else if( highlight_direction_ == DARKER ) {

    if( highlight_index_ > 0 ) {

      highlight_index_ -= 0.1;
      _updateTrueHighlightIndex();

    } else highlight_direction_ = (highlight_) ? LIGHTER:NOHIGHLIGHT;

  }

  return highlight_direction_;

}

//__________________________________________________________
const TaskButton::Direction& TaskButton::updateDirection( void )
{
  if( direction_ == BIGGER && index_ < _pixmaps().maxSizeIndex() -1 ) index_++;
  else if( direction_ == SMALLER && index_ > 0 ) index_--;
  else direction_ = NONE;
  return direction_;
}

//___________________________________________________
void TaskButton::paint( QPainter& painter ) const
{

  Debug::Throw( "TaskButton::Paint.\n" );
  _drawPixmap( painter, index_ );
  if( show_name_ ) _drawText( painter, index_ );

}

//___________________________________________________
void TaskButton::_drawPixmap( QPainter& painter, const unsigned int& index ) const
{

  Debug::Throw( "TaskButton::_drawPixmap .\n" );

  // store pointer to pixmap to be drawn
  const QPixmap& pixmap( _pixmaps().get( index, attributes_.flag( TaskAttributes::IS_SEPARATOR ) ? 0:highlightIndex() ) );

  // center pixmap
  int h_offset( ( rect_.width() - pixmap.width() )/2 );
  int v_offset( ( rect_.height() - pixmap.height() )/2 );
  if( show_name_ ) v_offset -= text_size_.height()/2;

  // paint
  painter.drawPixmap( rect_.x() + h_offset, rect_.y() + v_offset, pixmap );

}

//___________________________________________________
void TaskButton::_drawText( QPainter& painter, const unsigned int& index ) const
{

  if( !foreground_color_.isValid() ) return;

  // this should go to the cache too. To speed things up
  // retrieve pixmap to assign text position
  QPixmap tmp( rect_.size() );
  tmp.fill( Qt::transparent );
  {

    QPainter painter( &tmp );
    const QPixmap& pixmap( _pixmaps().get( index, 0 ) );
    int y = ( rect_.height() + pixmap.height() - text_size_.height() )/2;

    painter.setPen( foreground_color_ );
    painter.drawText(
      tmp.rect().left(),
      y,
      tmp.width(),
      text_size_.height(),
      Qt::AlignCenter,
      attributes_.name() );

  }

  // blur
  if( shadow_offset_ > 0 && shadow_color_.isValid() )
  {
    QImage image( tmp.toImage() );
    TRANSPARENCY::Effects::shadowBlur(image, shadow_offset_, shadow_color_ );
    painter.drawImage( rect_.topLeft(), image);
  }

  painter.drawPixmap( rect_.topLeft(), tmp );

}

//_______________________________________________________________________________
void TaskButton::_updateTrueHighlightIndex( void )
{ true_highlight_index_ = (unsigned int)( ( cos( M_PI*(1-highlight_index_/(maximum_highlight_index_))) + 1 )*( maximum_highlight_index_ )/2 ); }
