
// $Id: TaskDialog.cpp,v 1.21 2009/09/09 04:14:51 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file TaskDialog.cc
  \brief edit task taskrmations
  \author Hugo Pereira
  \version $Revision: 1.21 $
  \date $Date: 2009/09/09 04:14:51 $
*/

#include <QLayout>
#include <QLabel>

#include "AnimatedLineEditor.h"
#include "BrowsedLineEditor.h"
#include "GridLayout.h"
#include "TextEditor.h"
#include "Debug.h"
#include "IconBrowsedButton.h"
#include "TaskDialog.h"

using namespace std;

//______________________________________________
TaskDialog::TaskDialog( QWidget* parent ):
    CustomDialog( parent )
{

  Debug::Throw( "TaskDialog::TaskDialog.\n" );
  setOptionName( "TASK_DIALOG" );
  setWindowTitle( "Edit Button - Taskbar" );

  QHBoxLayout* h_layout = new QHBoxLayout();
  h_layout->setMargin(0);
  h_layout->setSpacing(5);
  mainLayout().addLayout( h_layout );

  QVBoxLayout* v_layout;
  h_layout->addLayout( v_layout = new QVBoxLayout() );
  v_layout->setMargin(10);

  v_layout->addWidget( icon_edit_ = new IconBrowsedButton( this ) );
  icon_edit_->setFile( TaskAttributes().defaultIconFilename(), false );
  v_layout->addStretch(1);

  h_layout->addLayout( v_layout = new QVBoxLayout() );
  v_layout->setMargin(0);
  v_layout->setSpacing(5);

  v_layout->addWidget( new QLabel( "Name:", this ) );
  v_layout->addWidget( name_line_edit_ = new AnimatedLineEditor( this ) );
  name_line_edit_->setToolTip(
    "Button name.\n"
    "It is displayed in tooltip and"
    "possibly in the task bar." );

  v_layout->addWidget( show_name_checkbox_ = new QCheckBox( "&Show button name", this ) );
  show_name_checkbox_->setToolTip(
    "Show this button name in taskbar.\n"
    "Note: this option overrides the\n "
    "global configuration settings." );

  // information layout
  mainLayout().addLayout( information_layout_ = new QVBoxLayout() );
  information_layout_->setMargin(0);
  information_layout_->setSpacing(5);

  // command
  mainLayout().addLayout( v_layout = new QVBoxLayout() );
  v_layout->setMargin(0);
  v_layout->setSpacing(5);

  QFrame* frame;
  v_layout->addWidget( frame = new QFrame( this ) );
  frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  GridLayout* grid_layout;
  v_layout->addLayout( grid_layout = new GridLayout() );
  grid_layout->setMargin(0);
  grid_layout->setSpacing(5);
  grid_layout->setMaxCount(2);

  grid_layout->addWidget( new QLabel( "Command:", this ) );
  grid_layout->addWidget( command_editor_ = new BrowsedLineEditor( this ) );
  command_editor_->setToolTip(
    "Command.\n"
    "Program executed in the background\n"
    "when clicking (left button) on the\n"
    "button.\n"
    "Add double-quotes around strings\n"
    "that contain white spaces" );

  // comments
  v_layout->addWidget( frame = new QFrame( this ) );
  frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  v_layout->addWidget( new QLabel( "Comments:", this ) );
  v_layout->addWidget( comments_editor_ = new TextEditor( this ) );
  comments_editor_->setToolTip(
    "Button comments.\n"
    "They are displayed in the tooltip." );

}

//________________________________________________________
TaskDialog& TaskDialog::setAttributes( const TaskAttributes& attributes )
{

  Debug::Throw( "TaskDialog::setAttributes.\n" );

  // store attributes
  attributes_ = attributes;
  if( !attributes_.creation().isValid() )
  {
    if( attributes_.modification().isValid() || attributes_.lastCalled().isValid() ) attributes_.setCreation( std::min(  attributes_.modification(), attributes_.lastCalled() ) );
    else attributes_.setCreation( TimeStamp::now() );
  }

  // update widgets
  icon_edit_->setFile( attributes_.iconName(), false );
  name_line_edit_->setText( attributes_.name() );
  show_name_checkbox_->setChecked( attributes_.flag( TaskAttributes::SHOW_NAME ) );
  command_editor_->editor().setText( attributes_.command() );
  comments_editor_->setText( attributes_.comments() );

  // add attributes information
  QFrame* frame;
  information_layout_->addWidget( frame = new QFrame( this ) );
  frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  GridLayout *grid_layout = new GridLayout();
  grid_layout->setMargin(0);
  grid_layout->setSpacing(5);
  grid_layout->setMaxCount(2);
  grid_layout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
  information_layout_->addLayout( grid_layout );

  if( attributes_.creation().isValid() )
  {
    grid_layout->addWidget( new QLabel( "Created: ", this ) );
    grid_layout->addWidget( new QLabel( attributes_.creation().toString(), this ) );
  }

  if( attributes_.modification().isValid() )
  {
    grid_layout->addWidget( new QLabel( "Last modified: ", this ) );
    grid_layout->addWidget( new QLabel( attributes_.modification().toString(), this ) );
  }

  if( attributes_.lastCalled().isValid() )
  {
    grid_layout->addWidget( new QLabel( "Last called: ", this ) );
    grid_layout->addWidget( new QLabel( attributes_.lastCalled().toString(), this ) );
  }

  return *this;

}

//_______________________________________
const TaskAttributes& TaskDialog::attributes( void )
{

  attributes_.setName( name_line_edit_->text() );
  attributes_.setCommand( command_editor_->editor().text() );
  attributes_.setComments( comments_editor_->toPlainText() );
  attributes_.setIconName( icon_edit_->file() );
  attributes_.setFlag( TaskAttributes::SHOW_NAME, show_name_checkbox_->isChecked() );
  return attributes_;

}
