// $Id: TaskListConfiguration.cpp,v 1.18 2009/09/09 02:29:48 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
  \file TaskListConfiguration.cc
  \brief task list configuration
  \author Hugo Pereira
  \version $Revision: 1.18 $
  \date $Date: 2009/09/09 02:29:48 $
*/

#include <QLayout>

#include "IconEngine.h"
#include "Icons.h"
#include "InformationDialog.h"
#include "QuestionDialog.h"
#include "TaskDialog.h"
#include "TaskListConfiguration.h"
#include "XmlOptions.h"

using namespace std;

//______________________________________________________
TaskListConfiguration::TaskListConfiguration( QWidget* parent, const TaskAttributes::List& task_list ):
    QWidget( parent ),
    Counter( "TaskListConfiguration" )
{
  setLayout( new QHBoxLayout() );
  layout()->setSpacing(5);
  layout()->setMargin(5);

  list_ = new TreeView( this );
  _list().setModel( &model_ );
  _list().setSortingEnabled( false );
  _list().setAllColumnsShowFocus( true );
  _list().setSelectionMode( QAbstractItemView::ContiguousSelection );
  layout()->addWidget( list_ );

  connect( list_, SIGNAL( activated( const QModelIndex& ) ), SLOT( _edit() ) );

  // vbox for buttons
  QVBoxLayout* v_layout( new QVBoxLayout() );
  v_layout->setMargin( 0 );
  v_layout->setSpacing( 5 );
  layout()->addItem( v_layout );

  // add task
  QPushButton *button;
  v_layout->addWidget( button = new QPushButton( "&Add Button", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _add() ) );
  button->setIcon( IconEngine::get( ICONS::ADD ) );
  button->setToolTip( "Add a new task" );
  add_button_ = button;

  // add separator
  v_layout->addWidget( button = new QPushButton( "&Add Separator", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _addSeparator() ) );
  button->setIcon( IconEngine::get( ICONS::ADD ) );
  button->setToolTip( "Add a separator" );
  add_separator_ = button;

  // edit
  v_layout->addWidget( button = new QPushButton( "&Edit", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _edit() ) );
  button->setIcon( IconEngine::get( ICONS::RENAME ) );
  button->setToolTip( "Edit selected task" );
  edit_ = button;

  // remove
  v_layout->addWidget( button = new QPushButton( "&Remove", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _remove() ) );
  button->setToolTip( "Remove selected task" );
  button->setIcon( IconEngine::get( ICONS::REMOVE ) );
  remove_ = button;

  v_layout->addWidget( button = new QPushButton( "Move &Up", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _up() ) );
  button->setIcon( IconEngine::get( ICONS::UP ) );
  button->setToolTip( "Move up selected tasks" );
  move_up_ = button;

  v_layout->addWidget( button = new QPushButton( "Move &Down", this ) );
  connect( button, SIGNAL( clicked() ), this, SLOT( _down() ) );
  button->setIcon( IconEngine::get( ICONS::DOWN ) );
  button->setToolTip( "Move down selected tasks" );
  move_down_ = button;

  v_layout->addStretch(1);


  // set connections
  connect( _list().selectionModel(), SIGNAL( selectionChanged( const QItemSelection& , const QItemSelection& ) ), SLOT( _updateButtons() ) );
  connect( _list().selectionModel(), SIGNAL( currentChanged( const QModelIndex& , const QModelIndex& ) ), SLOT( _updateButtons() ) );

  // update buttons
  _updateButtons();

  // set attributes
  setTaskList( task_list );

}

//_____________________________________________________
void TaskListConfiguration::setTaskList( const TaskAttributes::List& task_list )
{

  Debug::Throw( "TaskListConfiguration::setTaskList.\n" );
  backup_task_list_ = task_list;
  model_.set( vector<TaskAttributes>( task_list.begin(), task_list.end() ) );
  _list().resizeColumnToContents( TaskAttributesModel::ICON );

}

//_____________________________________________________
TaskAttributes::List TaskListConfiguration::taskAttributesList( void )
{ return model_.get(); }


//_____________________________________________________
void TaskListConfiguration::apply( void )
{ emit configurationChanged( taskAttributesList() ); }

//_____________________________________________________
void TaskListConfiguration::cancel( void )
{
  setTaskList( backupTaskAttributesList() );
  emit configurationChanged( backupTaskAttributesList() );
}

//_________________________________________________________
void TaskListConfiguration::_updateButtons( void )
{

  Debug::Throw( "TaskListConfiguration::_updateButtons.\n" );

  // get selected indexes
  TaskAttributesModel::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
  bool has_selection( !selection.empty() );
  bool has_tasks( selection.size() - count_if( selection.begin(), selection.end(), TaskAttributes::IsSeparatorFTor() ) > 0 );

  remove_->setEnabled( has_selection );
  edit_->setEnabled( has_tasks );

  move_up_->setEnabled( has_selection && !_list().selectionModel()->isSelected( model_.index( 0, 0 ) ) );
  move_down_->setEnabled( has_selection && !_list().selectionModel()->isSelected( model_.index( model_.rowCount()-1, 0 ) ) );

}

//_________________________________________________________
void TaskListConfiguration::_add( void )
{

  Debug::Throw( "TaskListConfiguration::_Add.\n" );
  TaskDialog dialog( this );
  if( !dialog.exec() ) return;
  TaskAttributes attributes( dialog.attributes() );
  model_.add( attributes );

  // retrieve matching index select and make visible
  QModelIndex index( model_.index( attributes ) );
  _list().scrollTo( index );
  _list().selectionModel()->select( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );
  _list().selectionModel()->setCurrentIndex( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );

}

//_________________________________________________________
void TaskListConfiguration::_addSeparator( void )
{
  Debug::Throw( "TaskListConfiguration::_Add.\n" );
  TaskAttributes attributes;
  attributes.setFlag( TaskAttributes::IS_SEPARATOR, true );
  model_.add( attributes );

  // retrieve matching index select and make visible
  QModelIndex index( model_.index( attributes ) );
  _list().scrollTo( index );
  _list().selectionModel()->select( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );
  _list().selectionModel()->setCurrentIndex( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );

}

//_________________________________________________________
void TaskListConfiguration::_edit( void )
{

  Debug::Throw( "TaskListConfiguration::_Edit.\n" );
  TaskAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
  if( selection.empty() )
  {
    InformationDialog( this, "no item selected. <Edit> canceled" ).exec();
    return;
  }

  for(TaskAttributes::List::iterator iter = selection.begin(); iter != selection.end(); iter++ )
  {

    // skip if separator
    if( iter->flag( TaskAttributes::IS_SEPARATOR ) )
    { continue; }

    TaskDialog dialog( this );
    if( !dialog.setAttributes(*iter).exec() ) continue;

    QModelIndex index( model_.index( *iter ) );
    model_.replace( index, dialog.attributes() );

  }

  return;

}

//_________________________________________________________
void TaskListConfiguration::_remove( void )
{
  Debug::Throw( "TaskListConfiguration::_Remove.\n" );
  TaskAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
  if( selection.empty() )
  {
    InformationDialog( this, "no item selected. <Remove> canceled" ).exec();
    return;
  } else if( !QuestionDialog( this, "remove selected items ?" ).exec() ) return;

  // remove
  model_.remove( selection );

  return;

}

//_________________________________________________________
void TaskListConfiguration::_up( void )
{

  Debug::Throw( "TaskListConfiguration::_up.\n" );
  TaskAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
  if( selection.empty() ) return;

  // retrieve selected indexes in list and store in model
  QModelIndexList selected_indexes( _list().selectionModel()->selectedRows() );
  TaskAttributes::List selected_attributes( model_.get( selected_indexes ) );

  TaskAttributes::List current_attributes( taskAttributesList() );
  TaskAttributes::List new_attributes;

  for( TaskAttributes::List::const_iterator iter = current_attributes.begin(); iter != current_attributes.end(); iter++ )
  {

    // check if new list is not empty, current index is selected and last index is not.
    // if yes, move.
    if(
      !( new_attributes.empty() ||
      selected_indexes.indexOf( model_.index( *iter ) ) == -1 ||
      selected_indexes.indexOf( model_.index( new_attributes.back() ) ) != -1
      ) )
    {
      TaskAttributes last( new_attributes.back() );
      new_attributes.pop_back();
      new_attributes.push_back( *iter );
      new_attributes.push_back( last );
    } else new_attributes.push_back( *iter );

  }

  model_.set( new_attributes );

  // restore selection
  _list().selectionModel()->select( model_.index( selected_attributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
  for( TaskAttributes::List::const_iterator iter = selected_attributes.begin(); iter != selected_attributes.end(); iter++ )
  { _list().selectionModel()->select( model_.index( *iter ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

  return;

}

//_________________________________________________________
void TaskListConfiguration::_down( void )
{

  Debug::Throw( "TaskListConfiguration::_down.\n" );
  TaskAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
  if( selection.empty() ) return;

  // retrieve selected indexes in list and store in model
  QModelIndexList selected_indexes( _list().selectionModel()->selectedIndexes() );
  TaskAttributes::List selected_attributes( model_.get( selected_indexes ) );

  TaskAttributes::List current_attributes( TaskListConfiguration::taskAttributesList() );
  TaskAttributes::List new_attributes;

  for( TaskAttributes::List::reverse_iterator iter = current_attributes.rbegin(); iter != current_attributes.rend(); iter++ )
  {

    // check if new list is not empty, current index is selected and last index is not.
    // if yes, move.
    if(
      !( new_attributes.empty() ||
      selected_indexes.indexOf( model_.index( *iter ) ) == -1 ||
      selected_indexes.indexOf( model_.index( new_attributes.back() ) ) != -1
      ) )
    {

      TaskAttributes last( new_attributes.back() );
      new_attributes.pop_back();
      new_attributes.push_back( *iter );
      new_attributes.push_back( last );

    } else new_attributes.push_back( *iter );
  }

  model_.set( TaskAttributes::List( new_attributes.rbegin(), new_attributes.rend() ) );

  // restore selection
  _list().selectionModel()->select( model_.index( selected_attributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
  for( TaskAttributes::List::const_iterator iter = selected_attributes.begin(); iter != selected_attributes.end(); iter++ )
  { _list().selectionModel()->select( model_.index( *iter ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

  return;

}
